@@ -4,10 +4,6 @@ package Finance::Calendar;
 
 Finance::Calendar - represents the trading calendar.
 
-=head1 VERSION
-
-version 0.01
-
 =head1 SYNOPSIS
 
     use Finance::Calendar;
@@ -63,7 +59,7 @@ This class is responsible for providing trading times or holidays related inform
 
 use Moose;
 
-our $VERSION = 0.02;
+our $VERSION = '0.03';
 
 use List::Util qw(min max first);
 use Date::Utility;
@@ -95,7 +91,7 @@ has _cache => (
 sub _get_cache {
     my ($self, $method_name, $exchange, @dates) = @_;
 
-    return unless exists $self->_cache->{$method_name};
+    return undef unless exists $self->_cache->{$method_name};
 
     my $key = join "_", ($exchange->symbol, (map { $self->trading_date_for($exchange, $_)->epoch } @dates));
     return $self->_cache->{$method_name}{$key};
@@ -107,7 +103,7 @@ sub _set_cache {
     my $key = join "_", ($exchange->symbol, (map { $self->trading_date_for($exchange, $_)->epoch } @dates));
     $self->_cache->{$method_name}{$key} = $value;
 
-    return;
+    return undef;
 }
 
 =head1 METHODS - TRADING DAYS RELATED
@@ -128,7 +124,7 @@ sub trades_on {
     }
 
     my $really_when = $self->trading_date_for($exchange, $when);
-    my $result = (@{$exchange->trading_days_list}[$really_when->day_of_week] && !$self->is_holiday_for($exchange->symbol, $really_when)) ? 1 : 0;
+    my $result      = (@{$exchange->trading_days_list}[$really_when->day_of_week] && !$self->is_holiday_for($exchange->symbol, $really_when)) ? 1 : 0;
 
     $self->_set_cache($result, 'trades_on', $exchange, $when);
     return $result;
@@ -180,14 +176,15 @@ sub trade_date_after {
 
     my $date_next;
     my $counter = 1;
-    my $begin = $self->trading_date_for($exchange, $date);
+    my $begin   = $self->trading_date_for($exchange, $date);
 
     if (my $cache = $self->_get_cache('trade_date_after', $exchange, $begin)) {
         return $cache;
     }
 
-    # look forward at most 7 days. The next trading day could have span over a weekend with multiple consecutive holidays.
-    while (not $date_next and $counter <= 7) {
+    # look forward at most 11 days. The next trading day could have span over a weekend with multiple consecutive holidays.
+    # We chosed 11 due to the fact that the longest trading holidays we have got so far was 10 days(TSE).
+    while (not $date_next and $counter <= 11) {
         my $possible = $begin->plus_time_interval($counter . 'd');
         $date_next = $possible if $self->trades_on($exchange, $possible);
         $counter++;
@@ -315,14 +312,11 @@ Return true is exchange is open at the given epoch, false otherwise.
 
 sub is_open_at {
     my ($self, $exchange, $date) = @_;
-
     my $opening = $self->opening_on($exchange, $date);
-
-    return unless $opening;
-    return if $self->_is_in_trading_break($exchange, $date);
-    return 1 if (not $date->is_before($opening) and not $date->is_after($self->closing_on($exchange, $date)));
+    return undef if (not $opening or $self->_is_in_trading_break($exchange, $date));
+    return 1     if (not $date->is_before($opening) and not $date->is_after($self->closing_on($exchange, $date)));
     # if everything falls through, assume it is not open
-    return;
+    return undef;
 }
 
 =head2 seconds_since_open_at
@@ -416,7 +410,6 @@ trading days. Currently, this applies on Fridays only:
 
 =over 4
 
-=item * for the Jakarta exchange
 
 =item * for forex or metals
 
@@ -440,20 +433,9 @@ sub regularly_adjusts_trading_hours_on {
     if ($exchange->symbol eq 'FOREX' or $exchange->symbol eq 'METAL') {
         $changes = {
             'daily_close' => {
-                to   => '21h',
+                to   => '20h55m',
                 rule => $rule,
             }};
-    } elsif ($exchange->symbol eq 'JSC') {
-        # Jakarta has pauses for Friday prayers
-        $changes = {
-            'morning_close' => {
-                to   => '4h30m',
-                rule => $rule,
-            },
-            'afternoon_open' => {
-                to   => '7h',
-                rule => $rule
-            }};
     }
 
     return $changes;
@@ -496,7 +478,7 @@ Returns true if the exchange opens late on the given date.
 sub opens_late_on {
     my ($self, $exchange, $when) = @_;
 
-    return unless $self->trades_on($exchange, $when);
+    return undef unless $self->trades_on($exchange, $when);
 
     my $opens_late;
     my $listed = $self->_get_partial_trading_for($exchange, 'late_opens', $when);
@@ -528,7 +510,7 @@ sub seconds_of_trading_between_epochs {
 
     # step 1: calculate non-cached incomplete start-day and end_dates
     my $day_start = $start_epoch - ($start_epoch % $full_day);
-    my $day_end   = $end_epoch -   ($end_epoch % $full_day);
+    my $day_end   = $end_epoch - ($end_epoch % $full_day);
     if (($day_start != $start_epoch) && ($start_epoch < $end_epoch)) {
         $result += $self->_computed_trading_seconds($exchange, $start_epoch, min($day_start + 86399, $end_epoch));
         $start_epoch = $day_start + $full_day;
@@ -558,9 +540,9 @@ Returns a Date::Utility object on a trading day where the exchange does not clos
 sub regular_trading_day_after {
     my ($self, $exchange, $when) = @_;
 
-    return if $self->closing_on($exchange, $when);
+    return undef if $self->closing_on($exchange, $when);
 
-    my $counter = 0;
+    my $counter             = 0;
     my $regular_trading_day = $self->trade_date_after($exchange, $when);
     while ($counter <= 10) {
         my $possible = $regular_trading_day->plus_time_interval($counter . 'd');
@@ -589,8 +571,8 @@ sub trading_period {
     my ($self, $exchange, $when) = @_;
 
     return [] if not $self->trades_on($exchange, $when);
-    my $open = $self->opening_on($exchange, $when);
-    my $close = $self->closing_on($exchange, $when);
+    my $open   = $self->opening_on($exchange, $when);
+    my $close  = $self->closing_on($exchange, $when);
     my $breaks = $self->trading_breaks($exchange, $when);
 
     my @times = ($open);
@@ -636,21 +618,10 @@ Is this exchange trading on daylight savings times for the given epoch?
 
 =cut
 
-{
-    my %cache;
+sub is_in_dst_at {
+    my ($self, $exchange, $epoch) = @_;
 
-    sub is_in_dst_at {
-        my ($self, $exchange, $epoch) = @_;
-
-        if (my $cache = $cache{$exchange->symbol}{$epoch}) {
-            return $cache;
-        }
-
-        my $date_object = Date::Utility->new($epoch);
-        $cache{$exchange->symbol}{$epoch} = $date_object->is_dst_in_zone($exchange->trading_timezone);
-
-        return $cache{$exchange->symbol}{$epoch};
-    }
+    return Date::Utility->new($epoch)->is_dst_in_zone($exchange->trading_timezone);
 }
 
 ### PRIVATE ###
@@ -662,13 +633,13 @@ sub _get_holidays_for {
     my $calendar = $self->calendar->{holidays};
     my $holiday  = $calendar->{$date};
 
-    return unless $holiday;
+    return undef unless $holiday;
 
     foreach my $holiday_desc (keys %$holiday) {
         return $holiday_desc if (first { $symbol eq $_ } @{$holiday->{$holiday_desc}});
     }
 
-    return;
+    return undef;
 }
 
 sub _is_in_trading_break {
@@ -721,12 +692,12 @@ Returns either C<undef>, a single L<Date::Utility>, or an arrayref of L<Date::Ut
 sub get_exchange_open_times {
     my ($self, $exchange, $date, $which) = @_;
 
-    my $when = (ref $date) ? $date : Date::Utility->new($date);
+    my $when          = (ref $date) ? $date : Date::Utility->new($date);
     my $that_midnight = $self->trading_date_for($exchange, $when);
     my $requested_time;
     if ($self->trades_on($exchange, $that_midnight)) {
         my $dst_key = $self->_times_dst_key($exchange, $that_midnight);
-        my $ti = $exchange->market_times->{$dst_key}->{$which};
+        my $ti      = $exchange->market_times->{$dst_key}->{$which};
         my $extended_lunch_hour;
         if ($which eq 'trading_breaks') {
             my $extended_trading_breaks = $exchange->market_times->{$dst_key}->{day_of_week_extended_trading_breaks};
@@ -763,14 +734,14 @@ sub _get_partial_trading_for {
     my $date            = $when->truncate_to_day->epoch;
     my $partial_defined = $cached->{$date};
 
-    return unless $partial_defined;
+    return undef unless $partial_defined;
 
     foreach my $close_time (keys %{$cached->{$date}}) {
         my $symbols = $cached->{$date}{$close_time};
         return $close_time if (first { $exchange->symbol eq $_ } @$symbols);
     }
 
-    return;
+    return undef;
 }
 
 sub _days_between {
@@ -818,11 +789,11 @@ sub _market_opens {
     my $date = $when;
     # Figure out which "trading day" we are on
     # even if it differs from the GMT calendar day.
-    my $next_day = $date->plus_time_interval('1d')->truncate_to_day;
+    my $next_day  = $date->plus_time_interval('1d')->truncate_to_day;
     my $next_open = $self->opening_on($exchange, $next_day);
     $date = $next_day if ($next_open and not $date->is_before($next_open));
 
-    my $open = $self->opening_on($exchange, $date);
+    my $open  = $self->opening_on($exchange, $date);
     my $close = $self->closing_on($exchange, $date);
 
     if (not $open) {
@@ -893,7 +864,9 @@ sub _market_opens {
                 my $int_close = $breaks[$i][1];
                 my $next_open = exists $breaks[$i + 1] ? $breaks[$i + 1][0] : $close;
 
-                if ($when->is_after($current_open) and $when->is_before($int_open)) {
+                if ($when->is_before($int_open)
+                    and ($when->is_same_as($current_open) or $when->is_after($current_open)))
+                {
                     return {
                         open   => 1,
                         opens  => undef,
@@ -926,7 +899,7 @@ sub _market_opens {
         }
     }
 
-    return;
+    return undef;
 }
 
 ## PRIVATE method _seconds_of_trading_between_epochs_days_boundary
@@ -939,7 +912,7 @@ sub _seconds_of_trading_between_epochs_days_boundary {
     my ($self, $exchange, $start_epoch, $end_epoch) = @_;
 
     my $cache_key = join('-', $exchange->symbol, $start_epoch, $end_epoch);
-    my $result = $cached_seconds_for_interval{$cache_key} //= do {
+    my $result    = $cached_seconds_for_interval{$cache_key} //= do {
         my $head = $self->_computed_trading_seconds($exchange, $start_epoch, $start_epoch + 86399);
         if ($end_epoch - $start_epoch > $full_day - 1) {
             my $tail = $self->_seconds_of_trading_between_epochs_days_boundary($exchange, $start_epoch + $full_day, $end_epoch);
@@ -982,7 +955,7 @@ sub _computed_trading_seconds {
             for my $break_interval (@{$breaks}) {
                 my $interval_open  = $break_interval->[0];
                 my $interval_close = $break_interval->[1];
-                my $close_am       = min($interval_open->epoch, $closing_epoch);
+                my $close_am       = min($interval_open->epoch,  $closing_epoch);
                 my $open_pm        = min($interval_close->epoch, $closing_epoch);
 
                 $total_lunch_break_time = max(min($open_pm, $end), $close_am) - min(max($close_am, $start), $open_pm);
